/*
 *  subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *  Contact: kitone at free dot fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author kitone (kitone at free dot fr)
 */


#include "Document.h"
#include "SubtitleSSA.h"
#include "SubtitleASS.h"
#include "SubtitleSubRip.h"
#include "SubtitleMicroDVD.h"
#include "Config.h"
#include <iostream>
#include "utility.h"

/*
 *	constructeur
 *	s'il y a une option dans le fichier config [encodings] default=xxxx
 *	on l'utilise pour choisir le charset, sinon on utilise UTF-8
 */
Document::Document()
{
	Config &cfg = Config::getInstance();
	Glib::ustring encoding;
	if(cfg.get_value_string("encodings", "default", encoding))
		m_charset = encoding;
	else
		m_charset = "UTF-8";
	
	m_subtitleModel = Glib::RefPtr<SubtitleModel>(new SubtitleModel);

	m_styleModel = Glib::RefPtr<StyleModel>(new StyleModel);

	//m_nameModel = Glib::RefPtr<NameModel>(new NameModel);
}

/*
 *	constructeur par copy
 */
Document::Document(Document &src)
{
	m_subtitleModel = Glib::RefPtr<SubtitleModel>(new SubtitleModel);
	m_styleModel = Glib::RefPtr<StyleModel>(new StyleModel);

	g_return_if_fail(m_subtitleModel);
	g_return_if_fail(m_styleModel);

	m_charset = src.m_charset;
	m_scriptInfo = src.m_scriptInfo;

	filenameDocument = src.filenameDocument;
	filenameMovie = src.filenameDocument;

	m_subtitleModel->copy(src.get_subtitle_model());

	m_styleModel->copy(src.get_style_model());
}

/*
 *	destructeur
 */
Document::~Document()
{
}

/*
 *	utilise simplement l'extention pour determiner quel type de fichier
 *	"srt" "ass" ou "ssa"
 *	en cas d'erreur, retourne ssa
 */
Glib::ustring Document::getFormat()
{
	if(filenameDocument.find(".ssa") != Glib::ustring::npos)
		return "ssa";
	else if(filenameDocument.find(".ass") != Glib::ustring::npos)
		return "ass";	
	else if(filenameDocument.find(".srt") != Glib::ustring::npos)
		return "srt";
	else if(filenameDocument.find(".sub") != Glib::ustring::npos)
		return "sub";
	
	return "ssa";
}

/*
 *	retourne le charset (encodage) du document
 *	a permet de sauvegarder dans le meme encodage qu'a la lecture
 *	...
 */
Glib::ustring Document::getCharset()
{
	return m_charset;
}
	
/*
 *	efface les informations :
 *	 - script info
 *	 - subtitle model
 *	 - style model
 *	
 *	reinitialize le charset
 */
void Document::clear()
{
	Config &cfg = Config::getInstance();
	Glib::ustring encoding;
	if(cfg.get_value_string("encodings", "default", encoding))
		m_charset = encoding;
	else
		m_charset = "UTF-8";

	m_scriptInfo = ScriptInfo();
	
	filenameDocument.clear();
	filenameMovie.clear();
	
	m_subtitleModel->clear();
	m_styleModel->clear();
	//m_nameModel->clear();
}

/*
 *	encodings ""=auto, "UTF-8", ...
 *	clear permet d'effacer l'ancien model, sinon on ajoute a la suite du document	
 */
bool Document::open(const Glib::ustring &filename, const Glib::ustring &encoding, bool clear_doc)
{
	try
	{
		if(_open(filename, encoding, clear_doc))
			return true;
		else
		{
			dialog_error(_("Open Document Failed."), "");
		}
	}
	catch(Glib::ConvertError &ex)
	{
		dialog_error(_("Open Document Failed."), ex.what());
	}

	return false;
}

/*
 *
 */
bool Document::_open(const Glib::ustring &filename, const Glib::ustring &encoding, bool clear_doc)
{
	if(clear_doc)
		clear();
	
	Glib::ustring charset;
	
	if(filename.find(".ssa") != Glib::ustring::npos)
	{
		SubtitleSSA ssa(this);
		if(!ssa.open(filename, encoding))
			return false;

		charset = ssa.get_charset();
	}
	else if(filename.find(".ass") != Glib::ustring::npos)
	{
		try
		{
			SubtitleASS ass(this);
			if(!ass.open(filename, encoding))
				return false;
			
			charset = ass.get_charset();

		}catch(Glib::Error &ex)
		{
			std::cerr << ex.what() << std::endl;
		}
	}
	else if(filename.find(".srt") != Glib::ustring::npos)
	{
		SubtitleSubRip srt(this);
		if(!srt.open(filename,encoding))
			return false;
		
		charset = srt.get_charset();
	}
	else if(filename.find(".sub") != Glib::ustring::npos)
	{
		SubtitleMicroDVD mdvd(this);
		if(!mdvd.open(filename, encoding))
			return false;
		
		charset = mdvd.get_charset();
	}
	else
	{
		std::cerr << "Document::open > failed" << std::endl;
		return false;
	}

	if(clear_doc)
	{
		m_charset = charset;
		filenameDocument = filename;
	}
	return true;
}

/*
 *	encodings ""=auto, "UTF-8", ...
 *	rename, change le nom du document (filenameDocument)
 */
bool Document::save(const Glib::ustring &filename, const Glib::ustring &encoding, bool clear_doc)
{
	try
	{
		if(_save(filename, encoding, clear_doc))
			return true;
		else
		{
			dialog_error(_("Save Document Failed."), "");
		}
	}
	catch(Glib::ConvertError &ex)
	{
		dialog_error(_("Save Document Failed."), ex.what());
	}

	return false;
}

/*
 *
 */
bool Document::_save(const Glib::ustring &filename, const Glib::ustring &encoding, bool rename)
{
	Glib::ustring charset=encoding;

	//
	if(!charset.empty() && rename)
		m_charset = charset;
	//
	if(encoding.empty())
		charset = getCharset();

	
	if(rename)
		filenameDocument = filename;

	if(filename.find(".ssa") != Glib::ustring::npos)
	{
		SubtitleSSA ssa(this);
		return ssa.save(filename, charset);
	}
	else if(filename.find(".ass") != Glib::ustring::npos)
	{
		SubtitleASS ass(this);
		return ass.save(filename, charset);
	}
	else if(filename.find(".srt") != Glib::ustring::npos)
	{
		SubtitleSubRip srt(this);
		return srt.save(filename,charset);
	}
	else if(filename.find(".sub") != Glib::ustring::npos)
	{
		SubtitleMicroDVD sub(this);
		return sub.save(filename, charset);
	}
	return false;
}

/*
 *	retourne le model pour la gestion des sous-titres
 *	on passe par un model (gtk) pour eviter d'avoir des doublons 
 *	dans les donnes et donc une grosse consomation memoire (inutile)
 */
Glib::RefPtr<SubtitleModel> Document::get_subtitle_model()
{
	return m_subtitleModel;
}

/*
 *	retourne le model pour la gestion des styles
 */
Glib::RefPtr<StyleModel> Document::get_style_model()
{
	return m_styleModel;
}

/*
 *	retourne les informations sur le script 
 *	principalement pour SSA/ASS
 */
ScriptInfo& Document::get_script_info()
{
	return m_scriptInfo;
}

